@@ -7,23 +7,37 @@ package Object::Trampoline;
 
 use strict;
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 
 use Carp;
 
 our $AUTOLOAD = '';
 
+########################################################################
+# the only purpose for this class is having an autoload
+# that blesses things into O::T::Bounce.
+#
+# there may not be any arguments... either way i need
+# to make a lexical copy of the stack for use in the
+# closure.
+#
+# the closure delays actual contstruction until it is
+# dereferenced in O::T::Bounce::AUTOLOAD.
+#
+# $sub is syntatic sugar but is inexpensive enough to
+# construct.
+
 AUTOLOAD
 {
-    # this class is meaningless, since the object
-    # will always be blessed into O::T::Bounce
-    # anyway. the call that got us here is useful
-    # since it is the destination class' const.
-    #
-    # $sub is syntatic sugar but is inexpensive
-    # enough to construct.
+    # discard this class, then grab the "real" class
+    # and its arguments and the constructor's name.
 
-    my ( $ignore, $class, @argz ) = @_;
+    shift;
+
+    my $class = shift
+    or croak "Bogus Object::Trampoline: missing destination class";
+
+    my @argz = @_;
 
     my $name = ( split /::/, $AUTOLOAD )[ -1 ];
 
@@ -54,10 +68,26 @@ AUTOLOAD
 
     my $name = ( split /::/, $AUTOLOAD )[ -1 ];
 
-    my $sub = eval { $class->can( $name ) }
-    or croak "Bogus method call: '$class' cannot '$name': $@";
+    # if the class implements te method directly
+    # then goto is more effecient; otherwise use
+    # the symbol table to dispatch it -- which 
+    # will end up back here on the way through
+    # the first time the method is called.
 
-    goto &$sub
+    if( my $sub = eval { $class->can( $name ) } )
+    {
+        goto &$sub
+    }
+    else
+    {
+        my $obj = shift;
+
+        $obj->$name( @_ );
+
+        # you'll end up here afater the call if
+        # you're in the debugger stepping into
+        # things...
+    }
 }
 
 # stub destroy is necessary to dodge AUTOLOAD for
@@ -202,6 +232,20 @@ Also not really a bug, but it is the caller's responsability
 to actually "use" or "require" the destination class prior
 to actually constructing the object.
 
+=item
+
+Another not-really-a-bug: if a trampoline object 
+is initially shared via the symbol table then 
+the $_[0] trick used here will create multiple 
+instances of the object. In some cases this is
+exactly what is required; in others (say an IRC
+connection object that joins a chat) the multiple
+objects instantiated will cause problems.
+
+At the moment I'm trying to find a graceful way 
+of replacing the object contents without knowing
+the base data type, which may lead me into the 
+dungeons of XS...  suggeions welcome :-)
 
 =back
 
